package aokai.app.BMSPayer;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

/**
 * データベース処理クラス
 */
public class DatabaseOpenHelper extends SQLiteOpenHelper {
	private final static String DB_NAME = "music.db";
	public final static String DB_TABLE = "music_List";

	/**
	 * コンストラクタ
	 */
	public DatabaseOpenHelper(Context context) {
		// 指定したデータベース名が存在しない場合は、新たに作成されonCreate()が呼ばれる
		// バージョンを変更するとonUpgrade()が呼ばれる
		super(context, DB_NAME, null, 1);
	}

	/**
	 * データベースの生成に呼び出されるので、 スキーマの生成を行う
	 */
	@Override
	public void onCreate(SQLiteDatabase db) {
		// テーブルの生成
		StringBuilder createSql = new StringBuilder();
		createSql.append("create table " + DB_TABLE + " (");
		createSql.append("ARTIST" + " text,");
		createSql.append("BPM" + " REAL,");
		createSql.append("DIFFICULTY" + " integer,");
		createSql.append("File_Format" + " text,");
		createSql.append("file_path" + " text,");
		createSql.append("file_name" + " text,");
		createSql.append("GENRE" + " text,");
		createSql.append("LNOBJ" + " integer,");
		createSql.append("LNTYPE" + " integer,");
		createSql.append("PLAYER" + " integer,");
		createSql.append("PLAYLEVEL" + " integer,");
		createSql.append("RANK" + " integer,");
		createSql.append("STAGEFILE" + " integer,");
		createSql.append("SUBTITLE" + " text,");
		createSql.append("TITLE" + " text,");
		createSql.append("TOTAL" + " REAL,");
		createSql.append("HIGHTSCORE" + " integer,");
		createSql.append("Max_Noat" + " integer,");
		createSql.append("CLEAR" + " integer,");
		createSql.append("MISS" + " integer,");
		createSql.append("MAXCOMBO " + " integer,");
		createSql.append("ONOFF" + " integer");
		createSql.append(")");
		db.execSQL(createSql.toString());
	}

	public void writeDB(SQLiteDatabase db, Read_HEADERFile header) {
		ContentValues values = new ContentValues();
		values.put("ARTIST", header.ARTIST);
		values.put("BPM", header.BPM);
		values.put("DIFFICULTY", header.DIFFICULTY);
		values.put("File_Format", header.File_Format);
		values.put("file_path", header.file_path);
		values.put("file_name", header.file_name);
		values.put("GENRE", header.GENRE);
		values.put("LNOBJ", header.LNOBJ);
		values.put("LNTYPE", header.LNTYPE);
		values.put("PLAYER", header.PLAYER);
		values.put("PLAYLEVEL", header.PLAYLEVEL);
		values.put("RANK", header.RANK);
		values.put("STAGEFILE", header.STAGEFILE);
		values.put("SUBTITLE", header.SUBTITLE);
		values.put("TITLE", header.TITLE);
		values.put("TOTAL", header.TOTAL);
		values.put("HIGHTSCORE", header.HIGHTSCORE);
		values.put("Max_Noat", header.Max_Noat);
		values.put("CLEAR", header.CLEAR);
		values.put("MISS", header.MISS);
		values.put("MAXCOMBO", header.MAXCOMBO);
		values.put("ONOFF", 1);
		db.insert(DB_TABLE, "", values);
	}

	public void AllDelete(SQLiteDatabase db) {
		db.delete(DB_TABLE, null, null);
	}

	/**
	 * idでBizCardをロードする
	 * 
	 * @param rowId
	 *            PK
	 * @return ロード結果
	 */
	public Read_HEADERFile load(SQLiteDatabase db, int rowId) {
		Read_HEADERFile header = null;
		String[] columns = { "ARTIST", "BPM", "DIFFICULTY", "File_Format",
				"file_path", "file_name", "GENRE", "LNOBJ", "LNTYPE", "PLAYER",
				"PLAYLEVEL", "RANK", "STAGEFILE", "SUBTITLE", "TITLE", "TOTAL",
				"HIGHTSCORE", "Max_Noat", "CLEAR", "MISS", "MAXCOMBO",};
		try {
			Cursor cursor = db.query(DB_TABLE, columns, "COLUMN_ID=" + "'"
					+ rowId + "'", null, null, null, null);
			cursor.moveToFirst();
			header = getRead_HEADER(cursor);
		} finally {
			db.close();
		}
		return header;
	}

	public Read_HEADERFile getRead_HEADER(Cursor cursor) {
		Read_HEADERFile header = new Read_HEADERFile();
		int temp = cursor.getInt(21);
		if(temp == 0){
			return null;
		}
		header.ARTIST = cursor.getString(0);
		header.BPM = cursor.getLong(1);
		header.DIFFICULTY = cursor.getInt(2);
		header.File_Format = cursor.getInt(3);
		header.file_path = cursor.getString(4);
		header.file_name = cursor.getString(5);
		header.GENRE = cursor.getString(6);
		if (cursor.getString(7) == "true") {
			header.LNOBJ = true;
		} else {
			header.LNOBJ = false;
		}
		if (cursor.getString(8) == "true") {
			header.LNTYPE = true;
		} else {
			header.LNTYPE = false;
		}
		header.PLAYER = cursor.getInt(9);
		header.PLAYLEVEL = cursor.getInt(10);
		header.RANK = cursor.getInt(11);
		if (cursor.getString(12) == "true") {
			header.STAGEFILE = true;
		} else {
			header.STAGEFILE = false;
		}
		header.SUBTITLE = cursor.getString(13);
		header.TITLE = cursor.getString(14);
		header.TOTAL = cursor.getLong(15);
		header.HIGHTSCORE = cursor.getInt(16);
		header.Max_Noat = cursor.getInt(17);
		header.CLEAR = cursor.getInt(18);
		header.MISS = cursor.getInt(19);
		header.MAXCOMBO = cursor.getInt(20);
		return header;
	}

	public void scoreUpdate(SQLiteDatabase db, Read_HEADERFile header,
			int score, int miss, int combo, int game_type, double gage_score,
			Boolean finish) {
		if (score > header.HIGHTSCORE) {
			ContentValues values = new ContentValues();
			values.put("HIGHTSCORE", score);
			String[] id = {header.file_name,
					"" + header.PLAYLEVEL };
			db.update(DB_TABLE, values,
					"file_name=? AND PLAYLEVEL=?", id);
		}
		if (miss < header.MISS || header.MISS == -1) {
			ContentValues values = new ContentValues();
			values.put("MISS", miss);
			String[] id = {header.file_name,
					"" + header.PLAYLEVEL };
			db.update(DB_TABLE, values,
					"file_name=? AND PLAYLEVEL=?", id);
		}
		if (combo > header.MAXCOMBO) {
			ContentValues values = new ContentValues();
			values.put("MAXCOMBO", combo);
			String[] id = {header.file_name,
					"" + header.PLAYLEVEL };
			db.update(DB_TABLE, values,
					"file_name=? AND PLAYLEVEL=?", id);
		}
		if (combo == header.Max_Noat) {
			// クリアランプをフルコンクリアーに
			ContentValues values = new ContentValues();
			values.put("CLEAR", 5);
			String[] id = {header.file_name,
					"" + header.PLAYLEVEL };
			db.update(DB_TABLE, values,
					"file_name=? AND PLAYLEVEL=?", id);
		} else {
			switch (game_type) {
			case 0:
				if (header.CLEAR < 3) {
					if (gage_score >= 80 && finish) {
						// クリアランプをノーマルクリアーに
						ContentValues values = new ContentValues();
						values.put("CLEAR", 3);
						String[] id = {header.file_name,
								"" + header.PLAYLEVEL };
						db.update(DB_TABLE, values,
								"file_name=? AND PLAYLEVEL=?",
								id);
					} else {
						// イージークリアがついてなく失敗なら失敗ランプ
						if (header.CLEAR != 2) {
							ContentValues values = new ContentValues();
							values.put("CLEAR", 1);
							String[] id = {header.file_name,
									"" + header.PLAYLEVEL };
							db.update(
									DB_TABLE,
									values,
									"file_name=? AND PLAYLEVEL=?",
									id);
						}
					}
				}
				break;
			case 1:
				if (header.CLEAR < 2) {
					if (gage_score >= 80&&finish) {
						// クリアランプをイージ～クリアーに
						ContentValues values = new ContentValues();
						values.put("CLEAR", 2);
						String[] id = {header.file_name,
								"" + header.PLAYLEVEL };
						db.update(DB_TABLE, values,
								"file_name=? AND PLAYLEVEL=?",
								id);
					} else {
						ContentValues values = new ContentValues();
						values.put("CLEAR", 1);
						String[] id = {header.file_name,
								"" + header.PLAYLEVEL };
						db.update(DB_TABLE, values,
								"file_name=? AND PLAYLEVEL=?",
								id);
					}
				}
				break;
			case 2:
				if (header.CLEAR < 4) {
					if (gage_score >= 2 && finish) {
						// クリアランプをハードクリアーに
						ContentValues values = new ContentValues();
						values.put("CLEAR", 4);
						String[] id = {header.file_name,
								"" + header.PLAYLEVEL };
						db.update(DB_TABLE, values,
								"file_name=? AND PLAYLEVEL=?",
								id);
					} else {
						if (header.CLEAR != 2 && header.CLEAR != 3) {
							ContentValues values = new ContentValues();
							values.put("CLEAR", 1);
							String[] id = {header.file_name,
									"" + header.PLAYLEVEL };
							db.update(
									DB_TABLE,
									values,
									"file_name=? AND PLAYLEVEL=?",
									id);
						}
					}
				}
			}
		}
	}

	/**
	 * データベースの更新
	 * 
	 * 親クラスのコンストラクタに渡すversionを変更したときに呼び出される
	 */
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// データベースの更新
		db.execSQL("drop talbe if exists " + DB_TABLE);
		onCreate(db);
	}
}
